
-- return the window size
getWindowSize = getWindowSize or function()
	return 727, 335
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()

	-- constants
	local kKnobSpace = 43
	local kBigKnobSpace = 101
	local kKnobSpaceV = 70
	local kLabelSpaceV = 30
	local kWideTextW = 57
	local kEnvW = 120
	local kEnvH = 70

	-- make groups and knobs
	--
	
	local x = 0

	theKnobRenderer = smallKnobRenderer

	inputsGroup = group( "Inputs" )
	local kInputsW = 40
	x = 0
	labelledDropDown( inputsGroup, "Input Thru", "Thru",		x, 0, kInputsW )
	
	inputToGatesGroup = group( "Input To Gates" )
	x = 0
	for i=1,8 do
		local param = "Input " .. i .. " To Gate"
		local textx = x
		inputToGatesGroup:add( knob( param, x + kWideTextW - kKnobSize, kTextFieldH+1, kKnobSize ) )
		inputToGatesGroup:add( textField( param, textx, 0, kWideTextW, kTextFieldH, "" ) )
		local lab = label( "" .. i, textx, kTextFieldH+1+kKnobSize-3, "left" )
		setLabelColour( lab )
		inputToGatesGroup:add( lab )

		x = x + kWideTextW + pad
	end

	miscGroup = group( "Misc" )
	x = 0
	smallKnobAndTextField( miscGroup, "Test Signal",	"Test",		x, 0 )
	miscGroup:setVisible( showTestSignals )

	interfaceGroup = group( "Interface" )
	x = 0
	labelledDropDown( interfaceGroup, "Interface Category", "Category",		x, 0 )

	oscGroup = group( "OSC" )
	smallKnobAndTextField( oscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	oscGroup:setVisible( false )

	outsGroup = group( "Outputs" )
	smallKnobAndTextField( outsGroup, "Output 1",		"1",		0*kKnobSpace, 0 )
	smallKnobAndTextField( outsGroup, "Output 2",		"2",		1*kKnobSpace, 0 )
	smallKnobAndTextField( outsGroup, "Output 3",		"3",		2*kKnobSpace, 0 )
	smallKnobAndTextField( outsGroup, "Output 4",		"4",		3*kKnobSpace, 0 )
	smallKnobAndTextField( outsGroup, "Output 5",		"5",		4*kKnobSpace, 0 )
	smallKnobAndTextField( outsGroup, "Output 6",		"6",		5*kKnobSpace, 0 )

	local kGatesKnobSpace = 45
	gatesGroup = group( "Gates" )
	for i=1,6 do
		smallKnobAndTextField( gatesGroup, "Gates " .. i .. " MIDI Channel",	i .. " Chan",	(2*i-2+0)*kGatesKnobSpace, 0 )
		smallKnobAndTextField( gatesGroup, "Gates " .. i .. " Base Note",		i .. " Base",	(2*i-2+1)*kGatesKnobSpace, 0, nil, "%.0f" )
	end

	local kTriggersKnobSize = 27
	local kTriggersKnobSpace = 35
	local kTriggersKnobSpaceV = 44
	local kTriggersFirstKnobX = 15

	local function triggersKnobAndTextField( group, param, x, y )
		knobAndTextField( group, param, nil, x, y, kTriggersKnobSize, kTextFieldH, nil, "%.0f" )
	end

	local function makeTriggersRow( g, id, v )
		local lab = label( id, 5, v+kTextFieldH, 10, kTextFieldH )
		setLabelColour( lab )
		g:add( lab )

		for i=1,8 do
			triggersKnobAndTextField( g, "Trigger " .. id .. "/" .. i .. " Length", kTriggersFirstKnobX+(i-1)*kTriggersKnobSpace, v )
		end
	end

	local function makeAccentsRow( g, id, v )
		local lab = label( id, 5, v+kTextFieldH, 10, kTextFieldH )
		setLabelColour( lab )
		g:add( lab )

		for i=1,4 do
			g:add( paramButton( "Accent " .. id .. "/" .. 2*i, id .. "/" .. 2*i, kTriggersFirstKnobX+(i-1)*kTriggersKnobSpace, v, kTriggersKnobSize, kTextFieldH ) )
		end
	end

	theKnobRenderer = tinyKnobRenderer

	triggersGroup = group( "Triggers" )
	for i=1,6 do
		makeTriggersRow( triggersGroup, "" .. i, (5-i)*kTriggersKnobSpaceV )
	end
	for i=1,8 do
		local lab = label( "" .. i, kTriggersFirstKnobX+(i-1)*kTriggersKnobSpace+0.5*kTriggersKnobSize, 5*kTriggersKnobSpaceV+kTextFieldH, kTriggersKnobSize, kTextFieldH )
		setLabelColour( lab )
		triggersGroup:add( lab )
	end
	triggersGroup:setVisible( false )

	theKnobRenderer = smallKnobRenderer

	local kAccentsSpaceV = kTextFieldH + 10
	accentsGroup = group( "Accents" )
	for i=1,6 do
		makeAccentsRow( accentsGroup, "" .. i, (5-i)*kAccentsSpaceV )
	end
	for i=1,4 do
		local lab = label( "" .. 2*i, kTriggersFirstKnobX+(i-1)*kTriggersKnobSpace+0.5*kTriggersKnobSize, 5*kAccentsSpaceV+kTextFieldH, kTriggersKnobSize, kTextFieldH )
		setLabelColour( lab )
		accentsGroup:add( lab )
	end
	accentsGroup:setVisible( false )

	midiGroup = group( "MIDI Out" )
	x = 0
	labelledDropDown( midiGroup, "MIDI Out MIDI Channel", "Channel",			x, kLabelSpaceV )
	labelledDropDown( midiGroup, "MIDI Out Channelise", "Channelise",			x, 0 )
	x = x + kDropdownW+10
	knobAndTextField( midiGroup, "MIDI Out Output",				"Output",		x, 0, kKnobSize, kTextFieldH, kWideTextW )
	x = x + kWideTextW + pad
	midiGroup:add( paramButton( "MIDI Out Clock", "Clock",						x, 2*kTextFieldH+12, kDropdownW, kTextFieldH ) )
	midiGroup:add( paramButton( "Use MIDI SPP", "SPP",							x, kTextFieldH+8, kDropdownW, kTextFieldH ) )
	labelledDropDown( midiGroup, "MIDI Clock Swing Type", "Swing",				x, 0 )
	x = x + kDropdownW + 10
	smallKnobAndTextField( midiGroup, "MIDI Clock Swing",		"Swing",		x, 0 )
	x = x + kKnobSpace
	smallKnobAndTextField( midiGroup, "MIDI Clock Offset",		"Offset",		x, 0 )
	x = x + kKnobSpace
	smallKnobAndTextField( midiGroup, "MIDI Out Polyphony",		"Poly",			x, 0 )
	x = x + kKnobSpace
	local kDelayButtonW = 80
	midiGroup:add( paramButton( "Match Note Offs", "NoteOffs",					x, 2*kTextFieldH+12, kDelayButtonW, kTextFieldH ) )
--[[
	smallKnobAndTextField( midiGroup, "Byte 1",		"1",		4*kKnobSpace, 0 )
	smallKnobAndTextField( midiGroup, "Byte 2",		"2",		5*kKnobSpace, 0 )
	smallKnobAndTextField( midiGroup, "Byte 3",		"3",		6*kKnobSpace, 0 )
	smallKnobAndTextField( midiGroup, "Bytes",		"Count",	7*kKnobSpace, 0 )
	midiGroup:add( paramButton( "Send", "Send",					8*kKnobSpace, 0, kKnobSize, kTextFieldH ) )
--]]
	midiGroup:add( paramButton( "Delay Start Slice", "Delay Start",	x, kTextFieldH+6, kDelayButtonW, kTextFieldH ) )
	midiGroup:add( paramButton( "Delay Stop Slice", "Delay Stop",	x, 0, kDelayButtonW, kTextFieldH ) )
	x = x + kDelayButtonW + 10

	-- positions!

	local xbase = windowEdgePad
	local ybase = windowEdgePad
	local x = xbase
	local y = ybase

	outsGroup:setPosition( x, y, true )
	x = x + outsGroup:width() + pad

	midiGroup:setPosition( x, y, true )
	x = x + midiGroup:width() + pad
	
	oscGroup:setPosition( x, y, true )
	x = x + oscGroup:width() + pad

	x = xbase
	y = y + outsGroup:height() + pad

	gatesGroup:setPosition( x, y, true )
	x = x + gatesGroup:width() + pad

	interfaceGroup:setPosition( x, y, true )

	x = xbase
	y = y + gatesGroup:height() + pad

	inputsGroup:setPosition( x, y, true )
	x = x + inputsGroup:width() + pad

	inputToGatesGroup:setPosition( x, y, true )
	x = x + inputToGatesGroup:width() + pad
	
	miscGroup:setPosition( x, y, true )
	x = x + miscGroup:width() + pad

	local prefsY = y + inputToGatesGroup:height() + pad

	x = xbase
	y = ybase
	triggersGroup:setPosition( x, y, true )
	x = x + triggersGroup:width() + pad
	y = y + triggersGroup:height() - accentsGroup:height()

	accentsGroup:setPosition( x, y, true )

	-- add directly positioned objects
	--
	
	local ww, wh = getWindowSize()
	local kButtonW = 78
	local kInfoW = 180
	x = ww - windowEdgePad - kButtonW
	y = prefsY

	triggersButton = luaButton( toggleTriggers, "Triggers", "Show/Hide Trigger Settings", "toggle", 
					x, y, kButtonW, 30 )
	add( triggersButton )
		
	x = x - kButtonW - pad
	add( propertyTriggerButton( 64094, "Don't Panic", "All gates/envs off", x, y, kButtonW, 30 ) )

	-- add the info display object
	x = x - kInfoW - pad
	theInfoDisplay = infoDisplay( x, y+10, kInfoW, 40, 6 )
	add( theInfoDisplay )

	if showLogicIndicators then
		local ww, wh = getWindowSize()
		local prefsW = kKnobSize+20
		local prefsH = 25
		local prefsX = ww-prefsW-windowEdgePad
		local logicH = 15
		add( propertyButton( 64140, "Logic", "In Logic Mode?", "indicator", prefsX, wh-prefsH-windowEdgePad-logicH-pad, prefsW, logicH ) )
	end

end		-- of createUI()

toggleTriggers = toggleTriggers or function()
	local v = triggersButton:getState()
	triggersGroup:setVisible( v )
	accentsGroup:setVisible( v )
	v = not v
	outsGroup:setVisible( v )
	gatesGroup:setVisible( v )
	midiGroup:setVisible( v )
	inputsGroup:setVisible( v )
	inputToGatesGroup:setVisible( v )
	theInfoDisplay:setVisible( v )
--	oscGroup:setVisible( v )
	redrawLabels()
end
