
-- return the window size
getWindowSize = getWindowSize or function()
	return 685, 360
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()

	-- constants
	local kKnobSpace = 60
	local kKnobSpaceV = 70
	local kTextW = 40
	local kPropertyTextFieldH = 20
	local kCalibrationW = 128
	local kCalibrationH = 60
	local kPrefsW = 60

	local function knobGroup( LR, title )
		g = group( title )
		smallKnobAndTextField( g, "Output Pitch " .. LR,		"Pitch",		0*kKnobSpace, 0, kTextW )
		smallKnobAndTextField( g, "Output Env " .. LR,			"Env",			1*kKnobSpace, 0, kTextW )
		smallKnobAndTextField( g, "Smoothing " .. LR,			"Smooth",		2*kKnobSpace, 0, kTextW )
		
		return g
	end

	-- make groups and knobs
	--

	theKnobRenderer = smallKnobRenderer

	miscGroup = group( "OSC" )
	smallKnobAndTextField( miscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	miscGroup:setVisible( false )

	local x

	followerGroup = group( "Follower" )
	x = 0
	labelledDropDown( followerGroup, "Input Source",			"Source",			x, 0*kKnobSpaceV )
	x = x + kDropdownW + 2*pad
	smallKnobAndTextField( followerGroup, "Highpass Cutoff",	"Highpass",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( followerGroup, "Lowpass Cutoff",		"Lowpass",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( followerGroup, "Env Gain",			"Env Gain",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( followerGroup, "Env Attack",			"Attack",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( followerGroup, "Env Release",		"Release",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( followerGroup, "Env Gate",			"Env Gate",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace

	pitchGroup = group( "Pitch" )
	x = 0
	labelledDropDown( pitchGroup, "Pitch Mode",				"Pitch Mode",		x, 0*kKnobSpaceV )
	x = x + kDropdownW + 2*pad
	smallKnobAndTextField( pitchGroup, "Volts Per Octave",	"V/Octave",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( pitchGroup, "Hz Per Volt",		"Hz/V",				x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	labelledDropDown( pitchGroup, "V Scale Mode",			"Scale Mode",		x, 0*kKnobSpaceV )
	x = x + kDropdownW + 2*pad
	smallKnobAndTextField( pitchGroup, "Manual V Scale",	"V Scale",			x, 0*kKnobSpaceV )
	x = x + kKnobSpace
	smallKnobAndTextField( pitchGroup, "Zero V Coarse",		"0V Coarse",		x, 0*kKnobSpaceV, nil, "%.0f" )
	x = x + kKnobSpace
	smallKnobAndTextField( pitchGroup, "Zero V Fine",		"0V Fine",			x, 0*kKnobSpaceV, nil, "%.1f" )
	x = x + kKnobSpace

	leftGroup = knobGroup( "L", "Left Outputs" )
	rightGroup = knobGroup( "R", "Right Outputs" )

	-- positions!

	local filenameY = windowEdgePad
	local calibrationY = windowEdgePad
	local xbase = windowEdgePad
	local ybase = filenameY + kPropertyTextFieldH + kTextFieldH + pad
	local x = xbase
	local y = ybase

	pitchGroup:setPosition( x, y, true )
	x = x + pitchGroup:width() + pad

	x = xbase
	y = y + pitchGroup:height() + pad
	
	followerGroup:setPosition( x, y, true )
	x = x + followerGroup:width() + pad
	
	miscGroup:setPosition( x, y, true )
	x = x + miscGroup:width() + pad

	x = xbase
	y = y + followerGroup:height() + pad
	
	leftGroup:setPosition( x, y, true )
	x = x + leftGroup:width() + pad
	
	rightGroup:setPosition( x, y, true )
	x = x + rightGroup:width() + pad

	-- add directly positioned objects
	--

	x = xbase
	x = x + pitchGroup:width() + pad
	y = calibrationY
	add( button( "VCLoad", "Load", x, y, kPrefsW, 28 ) )
	add( button( "Transfer", "Transfer", x, y+32, kPrefsW, 28 ) )
	local cd = calibrationDisplay( x + kPrefsW + pad, y, kCalibrationW, kCalibrationH )
	setCalibrationDisplayColours( cd )
	add( cd )

	local kInfoW = 200
	y = y + kCalibrationH + pad
	add( followerInfoDisplay( x, y, kInfoW, 30 ) )
	y = y + 30 + pad
	local envDisplay = followerEnvDisplay( x, y, kInfoW, 100 )
	setFollowerEnvelopeDisplayColours( envDisplay )
	add( envDisplay )
	y = y + 100 + pad
	local pitchDisplay = followerPitchDisplay( x, y, kInfoW, 100 )
	setFollowerPitchDisplayColours( pitchDisplay )
	add( pitchDisplay )

	x = pad
	local ww = xbase + pitchGroup:width()
	local kLabelW = 140
	local lab = label( "Calibration loaded from:", x+0.5*kLabelW, filenameY+0.5*(kPropertyTextFieldH+kTextFieldH)+1.5*kTextFieldH, kLabelW, kTextFieldH )
	setLabelColour( lab )
	add( lab )
	field = propertyTextField( 64121, "Loaded calibration file", x, filenameY, ww-2*pad, kPropertyTextFieldH )
	field:setEditable( false )
	add( field )

end		-- of createUI()
