
-- return the window size
getWindowSize = getWindowSize or function()
	return 570, 380
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()
	setBannerContent( "nameOnly", 20, 200 )

	-- constants
	local kKnobSpace = 60
	local kOutputKnobSize = kTinyKnobSize
	local kOutputKnobTextSize = 33
	local kOutputKnobSpace = 40
	local kOutputKnobSpaceV = 44
	local kOutputFirstKnobX = 15
	local kPropertyTextFieldH = 20
	local kCalibrationW = 128
	local kPrefsW = 60

	local function outputKnobAndTextField( group, param, x, y, format )
		format = format or "%.2f"
		knobAndTextField( group, param, nil, x, y, kOutputKnobSize, kTextFieldH, kOutputKnobTextSize, format )
	end
	
	local function makeOutputRow( g, id, v )
		local lab = label( id, 5, v+kTextFieldH, 10, kTextFieldH )
		setLabelColour( lab )
		g:add( lab )

		outputKnobAndTextField( g, "Output " .. id .. " Offset", kOutputFirstKnobX+0*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Note", kOutputFirstKnobX+1*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Gate", kOutputFirstKnobX+2*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " X", kOutputFirstKnobX+3*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Y", kOutputFirstKnobX+4*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Z", kOutputFirstKnobX+5*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Smoothing", kOutputFirstKnobX+6*kOutputKnobSpace, v, "%.0f" )
	end

	local function makeOutputLabel( g, name, i, outputLabelX, outputLabelY )
		local lab = label( name, outputLabelX+i*kOutputKnobSpace, outputLabelY, "center", kOutputKnobSize, kTextFieldH )
		setLabelColour( lab )
		g:add( lab )
	end

	-- make groups and knobs
	--

	theKnobRenderer = smallKnobRenderer

	miscGroup = group( "OSC" )
	smallKnobAndTextField( miscGroup, "OSC Port Offset",	"Port",		0, 0 )
	miscGroup:setVisible( false )

	voiceGroup = group( "Voice" )
	smallKnobAndTextField( voiceGroup, "Voice ID",	"ID",		0, 0 )
	voiceGroup:add( paramButton( "Gate Z",	"Gate Z",	kKnobSpace, 0, 60, 20 ) )

	theKnobRenderer = tinyKnobRenderer

	outputGroup = group( "Output Matrix" )
	makeOutputRow( outputGroup, "1", 5*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "2", 4*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "3", 3*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "4", 2*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "5", 1*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "6", 0*kOutputKnobSpaceV )
	local outputLabelY = 6*kOutputKnobSpaceV
	local outputLabelX = kOutputFirstKnobX
	makeOutputLabel( outputGroup, "Offset", 0, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Note", 1, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Gate", 2, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "X", 3, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Y", 4, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Z", 5, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Smth", 6, outputLabelX, outputLabelY )

	-- positions!

	local xbase = windowEdgePad
	local ybase = windowEdgePad
	local x = xbase
	local y = ybase

	voiceGroup:setPosition( x, y, true )
	x = x + voiceGroup:width() + pad
	local buttonsY = y + voiceGroup:height() + pad

	x = xbase + kPrefsW + pad + kCalibrationW + pad
	outputGroup:setPosition( x, y, true )
	local filenameY = y + outputGroup:height() + pad
	x = x + outputGroup:width() + pad

	miscGroup:setPosition( x, y, true )

	-- add directly positioned objects
	--
	
	x = xbase
	y = buttonsY

	add( button( "VCLoad", "Load", x, y, kPrefsW, 30 ) )
	y = y + 40

	add( button( "Transfer", "Transfer", x, y, kPrefsW, 30 ) )
	y = y + 40

	add( propertyTriggerButton( 64090, "ES-4", "Create a calibration suitable for the ES-4", x, y, kPrefsW, 30 ) )
	y = y + 40

	add( propertyTriggerButton( 64091, "ESX-xCV", "Create a calibration suitable for the ESX-4CV/8CV", x, y, kPrefsW, 30 ) )
	y = y + 40

	local cd = calibrationDisplay( x + kPrefsW + pad, buttonsY, kCalibrationW, 100 )
	setCalibrationDisplayColours( cd )
	add( cd )

	x = pad
	local ww, wh = getWindowSize()
	local kLabelW = 140
	local lab = label( "Calibration loaded from:", x, filenameY+kPropertyTextFieldH, "center", kLabelW, kTextFieldH )
	setLabelColour( lab )
	add( lab )
	field = propertyTextField( 64121, "Loaded calibration file", x, filenameY, ww-2*pad, kPropertyTextFieldH )
	field:setEditable( false )
	add( field )

end		-- of createUI()
