
-- return the window size
getWindowSize = getWindowSize or function()
	return 925, 385
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()

	-- constants
	local kKnobSpace = 60
	local kKnobSpaceV = 70
	local kOutputKnobSize = kTinyKnobSize
	local kOutputKnobTextSize = 33
	local kOutputKnobSpace = 40
	local kOutputKnobSpaceV = 44
	local kOutputFirstKnobX = 15
	local kTextFieldHSpace = 20
	local kCalibrationW = 128
	local kButtonW = kCalibrationW
	local kButtonH = 27
	local kButtonSpace = 31
    local kPropertyTextFieldH = 20

	local function outputKnobAndTextField( group, param, x, y, format )
		format = format or "%.2f"
		knobAndTextField( group, param, nil, x, y, kOutputKnobSize, kTextFieldH, kOutputKnobTextSize, format )
	end

	local function shapeKnobAndTextField( group, param, x, y, format )
		format = format or "%.2f"
		knobAndTextField( group, param, nil, x, y, kOutputKnobSize, kTextFieldH, kOutputKnobTextSize, format )
	end

	local function makeEnvGroup( id )
		local g = group( "Env " .. id )
		
		local kKnobSpace = 45
		
		smallKnobAndTextField( g, "Env " .. id .. " Time A1", "T A1", 0*kKnobSpace, 2*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Time A2", "T A2", 1*kKnobSpace, 2*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Time A3", "T A3", 2*kKnobSpace, 2*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Time R1", "T R1", 3*kKnobSpace, 2*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Time R2", "T R2", 4*kKnobSpace, 2*kKnobSpaceV )

		smallKnobAndTextField( g, "Env " .. id .. " Level A1", "Lvl A1", 0.5*kKnobSpace, 0*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Level A2", "Lvl A2", 1.5*kKnobSpace, 0*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Sustain", "Sustain", 2.5*kKnobSpace, 0*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Level R1", "Lvl R1", 3.5*kKnobSpace, 0*kKnobSpaceV )

		local y = kKnobSpaceV - kTextFieldH
		shapeKnobAndTextField( g, "Env " .. id .. " Shape A1", 0*kKnobSpace, y)
		shapeKnobAndTextField( g, "Env " .. id .. " Shape A2", 1*kKnobSpace, y )
		shapeKnobAndTextField( g, "Env " .. id .. " Shape A3", 2*kKnobSpace, y )
		shapeKnobAndTextField( g, "Env " .. id .. " Shape R1", 3*kKnobSpace, y )
		shapeKnobAndTextField( g, "Env " .. id .. " Shape R2", 4*kKnobSpace, y )

		local y = kKnobSpaceV + kOutputKnobTextSize
		local h = kTextFieldH + 2
		g:add( paramButton( "Env " .. id .. " Type A1",	"Exp", 0*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type A2",	"Exp", 1*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type A3",	"Exp", 2*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type R1",	"Exp", 3*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type R2",	"Exp", 4*kKnobSpace, y, kKnobSize, h ) )
		
		return g
	end
	
	local function makeOutputRow( g, id, v )
		local lab = label( id, 5, v+kTextFieldH, 10, kTextFieldH )
		setLabelColour( lab )
		g:add( lab )

		outputKnobAndTextField( g, "Output " .. id .. " Offset", kOutputFirstKnobX+0*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Pitch", kOutputFirstKnobX+1*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Gate", kOutputFirstKnobX+2*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Env1", kOutputFirstKnobX+3*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Env2", kOutputFirstKnobX+4*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Env3", kOutputFirstKnobX+5*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Trigger", kOutputFirstKnobX+6*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Velocity", kOutputFirstKnobX+7*kOutputKnobSpace, v )
		outputKnobAndTextField( g, "Output " .. id .. " Smoothing", kOutputFirstKnobX+8*kOutputKnobSpace, v, "%.0f" )
	end

	local function makeOutputLabel( g, name, i, outputLabelX, outputLabelY )
		local lab = label( name, outputLabelX+i*kOutputKnobSpace, outputLabelY, "center", kOutputKnobSize, kTextFieldH )
		setLabelColour( lab )
		g:add( lab )
	end

	-- make groups and knobs
	--

	theKnobRenderer = smallKnobRenderer

	oscGroup = group( "OSC" )
	smallKnobAndTextField( oscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	oscGroup:setVisible( false )

	miscGroup = group( "Modes" )
	miscGroup:add( dropDown( "Note Priority", 0, 2*kTextFieldHSpace, kDropdownW, kTextFieldH ) )
	miscGroup:add( dropDown( "Retrigger Mode", 0, 1*kTextFieldHSpace, kDropdownW, kTextFieldH ) )
	miscGroup:add( dropDown( "Poly Mode", 0, 0*kTextFieldHSpace, kDropdownW, kTextFieldH ) )
	smallKnobAndTextField( miscGroup, "Polyphony/Voice", "Ply/Vox",	kDropdownW+5, 0 )

	portaGroup = group( "Portamento" )
	labelledDropDown( portaGroup, "Portamento Mode", "Mode", 0, 0*kTextFieldHSpace, 66 )
	smallKnobAndTextField( portaGroup, "Portamento Time", "Time",	66+5, 0 )
	smallKnobAndTextField( portaGroup, "Portamento Shape", "Shape",	66+38+5, 0 )

	voltageGroup = group( "Voltages" )
	local kVoltageTextW = 40
	knobAndTextField( voltageGroup, "Gate Closed Voltage",	"Gate Clsd",	0*kKnobSpace, 0*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )
	knobAndTextField( voltageGroup, "Gate Open Voltage",	"Gate Open",	0*kKnobSpace, 1*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )
	knobAndTextField( voltageGroup, "Trigger Off Voltage",	"Trig Off",		1*kKnobSpace, 0*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )
	knobAndTextField( voltageGroup, "Trigger On Voltage",	"Trig On",		1*kKnobSpace, 1*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )
	knobAndTextField( voltageGroup, "Velocity Min Voltage",	"Vel Min",		2*kKnobSpace, 0*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )
	knobAndTextField( voltageGroup, "Velocity Max Voltage",	"Vel Max",		2*kKnobSpace, 1*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )

	overrideGroup = group( "Overrides" )
	overrideGroup:add( paramButton( 		"Override Pitch",	"Pitch",			0*kKnobSpace, 22, kKnobSpace-pad, 25 ) )
	overrideGroup:add( paramButton( 		"Override Gate",	"Gate",				0*kKnobSpace, -10, kKnobSpace-pad, 25 ) )
	smallKnobAndTextField( overrideGroup, "Pitch",			"Pitch",				1*kKnobSpace, 0 )
	smallKnobAndTextField( overrideGroup, "Gate",			"Gate",					2*kKnobSpace, 0 )

	detuneGroup = group( "Detune" )
	smallKnobAndTextField( detuneGroup, "Pitch Wheel Range",	"Bend Rng",		0*kKnobSpace, 0 )
	knobAndTextField( detuneGroup, "Transpose",			"Transpose",	1*kKnobSpace, 0, kKnobSize, kTextFieldH, 40 )
	smallKnobAndTextField( detuneGroup, "Detune",				"Detune",		2*kKnobSpace, 0 )
	smallKnobAndTextField( detuneGroup, "Random Tuning",		"Random",		3*kKnobSpace, 0 )
	smallKnobAndTextField( detuneGroup, "Pitch LFO Depth",		"LFO Dpth",		4*kKnobSpace, 0 )
	smallKnobAndTextField( detuneGroup, "Pitch LFO Speed",		"LFO Spd",		5*kKnobSpace, 0 )
	
	envsGroup = group( "Simple Env Controls" )
	smallKnobAndTextField( envsGroup, "Env 1 Time A2", "Env 1 A",		0*kKnobSpace, 2*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 1 Time A3", "Env 1 D",		1*kKnobSpace, 2*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 1 Sustain", "Env 1 S",		2*kKnobSpace, 2*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 1 Time R1", "Env 1 R",		3*kKnobSpace, 2*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 2 Time A2", "Env 2 A",		0*kKnobSpace, 1*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 2 Time A3", "Env 2 D",		1*kKnobSpace, 1*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 2 Sustain", "Env 2 S",		2*kKnobSpace, 1*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 2 Time R1", "Env 2 R",		3*kKnobSpace, 1*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 3 Time A2", "Env 3 A",		0*kKnobSpace, 0*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 3 Time A3", "Env 3 D",		1*kKnobSpace, 0*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 3 Sustain", "Env 3 S",		2*kKnobSpace, 0*kKnobSpaceV )
	smallKnobAndTextField( envsGroup, "Env 3 Time R1", "Env 3 R",		3*kKnobSpace, 0*kKnobSpaceV )

	env1Group = makeEnvGroup( "1" )
	env2Group = makeEnvGroup( "2" )
	env3Group = makeEnvGroup( "3" )
	env1Group:setVisible( false )
	env2Group:setVisible( false )
	env3Group:setVisible( false )

	envVelocityGroup = group( "Env Misc" )
	local kEnvDepthTextW = 40
	knobAndTextField( envVelocityGroup, "Env 1 Velocity Depth", "Vel 1", 0*kKnobSpace, 2*kKnobSpaceV, kKnobSize, kTextFieldH, kEnvDepthTextW )
	knobAndTextField( envVelocityGroup, "Env 2 Velocity Depth", "Vel 2", 0*kKnobSpace, 1*kKnobSpaceV, kKnobSize, kTextFieldH, kEnvDepthTextW )
	knobAndTextField( envVelocityGroup, "Env 3 Velocity Depth", "Vel 3", 0*kKnobSpace, 0*kKnobSpaceV, kKnobSize, kTextFieldH, kEnvDepthTextW )
	local envMultSpace = kEnvDepthTextW + 10
	labelledDropDown( envVelocityGroup, "Env 1 Multiplier", "Mult 1", envMultSpace, 2*kKnobSpaceV, 35 )
	labelledDropDown( envVelocityGroup, "Env 2 Multiplier", "Mult 2", envMultSpace, 1*kKnobSpaceV, 35 )
	labelledDropDown( envVelocityGroup, "Env 3 Multiplier", "Mult 3", envMultSpace, 0*kKnobSpaceV, 35 )
	envVelocityGroup:add( paramButton( "Env 1 Reset To Zero",	"RTZ",	envMultSpace, 2*kKnobSpaceV+26, 35, 20 ) )
	envVelocityGroup:add( paramButton( "Env 2 Reset To Zero",	"RTZ",	envMultSpace, 1*kKnobSpaceV+26, 35, 20 ) )
	envVelocityGroup:add( paramButton( "Env 3 Reset To Zero",	"RTZ",	envMultSpace, 0*kKnobSpaceV+26, 35, 20 ) )

	theKnobRenderer = tinyKnobRenderer

	outputGroup = group( "Output Matrix" )
	makeOutputRow( outputGroup, "1", 5*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "2", 4*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "3", 3*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "4", 2*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "5", 1*kOutputKnobSpaceV )
	makeOutputRow( outputGroup, "6", 0*kOutputKnobSpaceV )
	local outputLabelY = 6*kOutputKnobSpaceV
	local outputLabelX = kOutputFirstKnobX
	makeOutputLabel( outputGroup, "Offset", 0, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Pitch", 1, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Gate", 2, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Env1", 3, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Env2", 4, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Env3", 5, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Trig", 6, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Vel", 7, outputLabelX, outputLabelY )
	makeOutputLabel( outputGroup, "Smth", 8, outputLabelX, outputLabelY )
	outputGroup:setVisible( false )

	targetGroup = group( "ESX-xCV Target" )
	x = 0
	y = 0
	labelledDropDown( targetGroup, "Target Hardware", "Hardware",	x, 0 )
	x = x + kDropdownW + pad
	labelledDropDown( targetGroup, "Target Channel", "Channel",		x, 0 )
	x = x + kDropdownW + pad
	targetGroup:setVisible( false )

	es4Group = group( "ES-4 Settings" )
	local kBipolarButtonH = 14
	x = 0
	y = 0
	es4Group:add( paramButton( "Bipolar 3", "Bipolar 3", x, y, kDropdownW, kBipolarButtonH ) )
	x = x + kDropdownW + pad
	es4Group:add( paramButton( "Bipolar 4", "Bipolar 4", x, y, kDropdownW, kBipolarButtonH ) )
	x = x + kDropdownW + pad
	es4Group:add( paramButton( "Bipolar 5", "Bipolar 5", x, y, kDropdownW, kBipolarButtonH ) )
	es4Group:setVisible( false )

	hwCalibrationGroup = group( "Calibration" )
	x = 0
	y = 0
	hwCalibrationGroup:add( propertyTriggerButton( 64091, "Create ESX-xCV Calibration", "Create a calibration suitable for the ESX-4CV/8CV", x, y, 3*kDropdownW+2*pad, kButtonH ) )
	y = y + kButtonH + pad
	hwCalibrationGroup:add( propertyTriggerButton( 64090, "Create ES-4 Calibration", "Create a calibration suitable for the ES-4", x, y, 3*kDropdownW+2*pad, kButtonH ) )
	y = y + kButtonH + pad
	hwCalibrationGroup:setVisible( false )
	
	triggerGroup = group( "Trigger" )
	smallKnobAndTextField( triggerGroup, "Trigger Length",	"Length",	0*kKnobSpace, 0 )

	interfaceGroup = group( "Interface" )
	x = 0
	y = 0
	labelledDropDown( interfaceGroup, "Interface Category", "Category",		x, y )
	interfaceGroup:setVisible( false )

	hardwareGroup = group( "Hardware" )
	x = 0
	y = 0
	labelledDropDown( hardwareGroup, "Hardware Mode", "Mode", x, y )
	hardwareGroup:setVisible( false )

    tuningGroup = group( "Tuning" )
    x = 0
    y = 0
    local kTuneButtonW = 80
    local kLabelW = 100
	local lab = label( "Status:", x, y+0.5*(kPropertyTextFieldH-kTextFieldH), "center", kLabelW, kTextFieldH )
	setLabelColour( lab )
	tuningGroup:add( lab )
	field = propertyTextField( 64127, "Status", x+kLabelW+pad, y, outputGroup:width()-kLabelW-3*pad, kPropertyTextFieldH )
	field:setEditable( false )
	tuningGroup:add( field )
	y = y + 30
    tuningGroup:add( button( "SclLoad", "Load .scl", x, y, kTuneButtonW, 30 ) )
    tuningGroup:add( button( "KbmLoad", "Load .kbm", x+kTuneButtonW+pad, y, kTuneButtonW, 30 ) )
    tuningGroup:add( paramButton( "Enable .scl Tuning", "Enable", x+2*kTuneButtonW+2*pad, y, kTuneButtonW, 30 ) )
    y = y + 30 + pad
    local lab = label( ".kbm loaded from:", x, y+0.5*(kPropertyTextFieldH-kTextFieldH), "center", kLabelW, kTextFieldH )
    setLabelColour( lab )
    tuningGroup:add( lab )
    field = propertyTextField( 64123, "Loaded .kbm file", x+kLabelW+pad, y, outputGroup:width()-kLabelW-3*pad, kPropertyTextFieldH )
    field:setEditable( false )
    tuningGroup:add( field )
    y = y + 30
    field = propertyTextField( 64124, ".scl description", x, y, outputGroup:width()-2*pad, kPropertyTextFieldH )
    field:setEditable( false )
    tuningGroup:add( field )
    y = y + 30
    local lab = label( ".scl loaded from:", x, y+0.5*(kPropertyTextFieldH-kTextFieldH), "center", kLabelW, kTextFieldH )
    setLabelColour( lab )
    tuningGroup:add( lab )
    field = propertyTextField( 64122, "Loaded .scl file", x+kLabelW+pad, y, outputGroup:width()-kLabelW-3*pad, kPropertyTextFieldH )
    field:setEditable( false )
    tuningGroup:add( field )
    tuningGroup:setVisible( false )

	-- positions!

	local xbase = windowEdgePad
	local ybase = windowEdgePad
	local x = xbase + kCalibrationW + pad
	local y = ybase
	
	detuneGroup:setPosition( x, y, true )
	local xbase2 = x

	x = x + detuneGroup:width() + pad
	local envControlsX = x - pad

	portaGroup:setPosition( x, y, true )
	x = x + portaGroup:width() + pad

	triggerGroup:setPosition( x, y - portaGroup:height() + triggerGroup:height(), true )
	x = x + triggerGroup:width() + pad

	oscGroup:setPosition( x, y, true )
	x = x + oscGroup:width() + pad

	x = xbase2
	y = y + detuneGroup:height() + pad
	
	overrideGroup:setPosition( x, y, true )
	y = y + overrideGroup:height() + pad
	
	voltageGroup:setPosition( x, y, true )

	x = x + voltageGroup:width() + pad
	local topy = y + voltageGroup:height()
	
	local envx = x
	local kEnvW = envControlsX - envx
	x = x + kEnvW + pad
	local by = topy - kTextFieldH

	y = by - pad
	local envsY = y - envsGroup:height()
	envsGroup:setPosition( x, envsY, true )
	env1Group:setPosition( x, envsY, true )
	env2Group:setPosition( x, envsY, true )
	env3Group:setPosition( x, envsY, true )
	outputGroup:setPosition( x, y-outputGroup:height(), true )
	local hwy = y + pad
	hwy = hwy - interfaceGroup:height() - pad
	interfaceGroup:setPosition( x, hwy, true )
	hardwareGroup:setPosition( x + interfaceGroup:width() + pad, hwy, true )
	hwy = hwy - targetGroup:height() - pad
	targetGroup:setPosition( x, hwy, true )
	hwy = hwy - es4Group:height() - pad
	es4Group:setPosition( x, hwy, true )
	hwy = hwy - hwCalibrationGroup:height() - pad
	hwCalibrationGroup:setPosition( x, hwy, true )
    tuningGroup:setPosition( x, y-tuningGroup:height(), true )

	local bx = x

	local kEnvButtonW = 53
	local kEnvButtonH = 14
	local kEnvButtonSpace = 55
	add( luaButton( envsButton, "Simple", "Show Simple Envelope Controls", "push", 
					bx, by, kEnvButtonW, kEnvButtonH ) )
	bx = bx + kEnvButtonSpace
	add( luaButton( env1Button, "Env 1", "Show Envelope 1 Controls", "push", 
					bx, by, kEnvButtonW, kEnvButtonH ) )
	bx = bx + kEnvButtonSpace
	add( luaButton( env2Button, "Env 2", "Show Envelope 2 Controls", "push", 
					bx, by, kEnvButtonW, kEnvButtonH ) )
	bx = bx + kEnvButtonSpace
	add( luaButton( env3Button, "Env 3", "Show Envelope 3 Controls", "push", 
					bx, by, kEnvButtonW, kEnvButtonH ) )
	bx = bx + kEnvButtonSpace
	add( luaButton( outputsButton, "Outputs", "Show Output Matrix", "push", 
					bx, by, kEnvButtonW, kEnvButtonH ) )
	bx = bx + kEnvButtonSpace
	add( luaButton( es4Button, "H/W", "Show Expert Sleepers Hardware Controls", "push", 
					bx, by, kEnvButtonW, kEnvButtonH ) )
	bx = bx + kEnvButtonSpace
    add( luaButton( tuningButton, "Tuning", "Show Tuning Controls", "push",
                    bx, by, kEnvButtonW, kEnvButtonH ) )
    bx = bx + kEnvButtonSpace

	x = x + envsGroup:width() + pad

	envVelocityGroup:setPosition( x, envsY, true )
	x = x + envVelocityGroup:width() + pad

	x = xbase
	y = ybase

	miscGroup:setPosition( x, y, true )

	-- add directly positioned objects
	--

	x = envx
	y = ybase + detuneGroup:height() + pad
	local kEnvH = math.floor( ( voltageGroup:height() + overrideGroup:height() + pad - 2*pad )/3 )
	
	for i=2,0,-1 do
		local env = envelopeDisplay( i, x, y, kEnvW, kEnvH )
		setEnvelopeDisplayColours( env )
		add( env )
		y = y + kEnvH + pad
	end
	y = y - ( kEnvH + pad )
	
	x = xbase

	local cd = calibrationDisplay( x, y, kCalibrationW, kEnvH )
	setCalibrationDisplayColours( cd )
	add( cd )
	
	y = y - 35

	add( propertyTriggerButton( 64092, "Calibrate V/oct", "Start calibration routine", x, y, kButtonW, kButtonH ) )
	y = y - kButtonSpace

	add( propertyTriggerButton( 64093, "Calibrate Hz/V", "Start calibration routine", x, y, kButtonW, kButtonH ) )
	y = y - kButtonSpace
	
	local w = ( kButtonW/2 ) - 5
	add( button( "VCLoad", "Load", x, y, w, kButtonH ) )
	add( button( "VCSave", "Save", x+w+10, y, w, kButtonH ) )
	y = y - kButtonSpace
	
	add( propertyTriggerButton( 64094, "Don't Panic", "All gates/envs off", x, y, kButtonW, kButtonH ) )
	y = y - kButtonSpace

end		-- of createUI()

envsButton = envsButton or function()
	envsGroup:setVisible( true )
	env1Group:setVisible( false )
	env2Group:setVisible( false )
	env3Group:setVisible( false )
	envVelocityGroup:setVisible( true )
	outputGroup:setVisible( false )
	es4Group:setVisible( false )
    tuningGroup:setVisible( false )
	targetGroup:setVisible( false )
	hwCalibrationGroup:setVisible( false )
	interfaceGroup:setVisible( false )
	hardwareGroup:setVisible( false )
	triggerGroup:setVisible( true )
	portaGroup:setVisible( true )
--	oscGroup:setVisible( true )
	redrawLabels()
end

env1Button = env1Button or function()
	envsGroup:setVisible( false )
	env1Group:setVisible( true )
	env2Group:setVisible( false )
	env3Group:setVisible( false )
	envVelocityGroup:setVisible( true )
	outputGroup:setVisible( false )
	es4Group:setVisible( false )
    tuningGroup:setVisible( false )
	targetGroup:setVisible( false )
	hwCalibrationGroup:setVisible( false )
	interfaceGroup:setVisible( false )
	hardwareGroup:setVisible( false )
	triggerGroup:setVisible( true )
	portaGroup:setVisible( true )
--	oscGroup:setVisible( true )
	redrawLabels()
end

env2Button = env2Button or function()
	envsGroup:setVisible( false )
	env1Group:setVisible( false )
	env2Group:setVisible( true )
	env3Group:setVisible( false )
	envVelocityGroup:setVisible( true )
	outputGroup:setVisible( false )
	es4Group:setVisible( false )
    tuningGroup:setVisible( false )
	targetGroup:setVisible( false )
	hwCalibrationGroup:setVisible( false )
	interfaceGroup:setVisible( false )
	hardwareGroup:setVisible( false )
	triggerGroup:setVisible( true )
	portaGroup:setVisible( true )
--	oscGroup:setVisible( true )
	redrawLabels()
end

env3Button = env3Button or function()
	envsGroup:setVisible( false )
	env1Group:setVisible( false )
	env2Group:setVisible( false )
	env3Group:setVisible( true )
	envVelocityGroup:setVisible( true )
	outputGroup:setVisible( false )
	es4Group:setVisible( false )
    tuningGroup:setVisible( false )
	targetGroup:setVisible( false )
	hwCalibrationGroup:setVisible( false )
	interfaceGroup:setVisible( false )
	hardwareGroup:setVisible( false )
	triggerGroup:setVisible( true )
	portaGroup:setVisible( true )
--	oscGroup:setVisible( true )
	redrawLabels()
end

outputsButton = outputsButton or function()
	envsGroup:setVisible( false )
	env1Group:setVisible( false )
	env2Group:setVisible( false )
	env3Group:setVisible( false )
	envVelocityGroup:setVisible( false )
	outputGroup:setVisible( true )
	es4Group:setVisible( false )
    tuningGroup:setVisible( false )
	targetGroup:setVisible( false )
	hwCalibrationGroup:setVisible( false )
	interfaceGroup:setVisible( false )
	hardwareGroup:setVisible( false )
	triggerGroup:setVisible( false )
	portaGroup:setVisible( false )
--	oscGroup:setVisible( false )
	redrawLabels()
end

es4Button = es4Button or function()
	envsGroup:setVisible( false )
	env1Group:setVisible( false )
	env2Group:setVisible( false )
	env3Group:setVisible( false )
	envVelocityGroup:setVisible( false )
	outputGroup:setVisible( false )
	es4Group:setVisible( true )
    tuningGroup:setVisible( false )
	targetGroup:setVisible( true )
	hwCalibrationGroup:setVisible( true )
	interfaceGroup:setVisible( true )
	hardwareGroup:setVisible( true )
	triggerGroup:setVisible( false )
	portaGroup:setVisible( false )
--	oscGroup:setVisible( false )
	redrawLabels()
end

tuningButton = tuningButton or function()
    envsGroup:setVisible( false )
    env1Group:setVisible( false )
    env2Group:setVisible( false )
    env3Group:setVisible( false )
    envVelocityGroup:setVisible( false )
    outputGroup:setVisible( false )
    es4Group:setVisible( false )
    tuningGroup:setVisible( true )
    targetGroup:setVisible( false )
    hwCalibrationGroup:setVisible( false )
    interfaceGroup:setVisible( false )
    hardwareGroup:setVisible( false )
    triggerGroup:setVisible( false )
    portaGroup:setVisible( false )
--  oscGroup:setVisible( false )
    redrawLabels()
end
